/***************************************************************************
Copyright (c) 2006 Peter Korzuszek, Damian Rakowski, KateOS Project
 
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
 
*Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.
 
*Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE. 
****************************************************************************/

#ifndef SMART_TOOLS_DIR_H
#define SMART_TOOLS_DIR_H

#include <string>
#include <iostream>
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <dirent.h>
#include "stassert.h"
#include "smarttools.h"
#include "ststring.h"
#include "stfile.h"
#include "stvaluelist.h"
#include "stassert.h"

/*!
   	\brief Allow operations on local files.
   			
   	Enable the creation of directories, get their permissions, and check if
   	the path exists, and whether or not it is a directory.
*/
class StDir 
{
	public:
		StDir(){}
		StDir( const StString& path ){ setPath( path ); }
		StDir( const char* path ){ setPath( path ); }
		
		/** Set the current path */
		void setPath( const StString& path ){ this->path = path; }
		
		/** Get the current path */
		StString getPath() const { return path; }
		
		StString getDirName() const;
		
		/*!
			Get the current file permissions.
			
			\return StPermission class represents file permission.
		*/ 
		StPermission getPermissions() const { StFile file( path ); return file.getPermissions(); }
		//TODO: setPermissions();
		
		/** Check if the current path exist*/
		bool exists() const;
		/** Check if the current path is a directory*/		
		bool isDirectory() const;
		
		/*!
			Create the directory 
			
			\param str - directory to create
			\param resursively - if directory should by created recursively
			
			\return True if the directory has created.
		*/ 
		bool create( const StString& str="", bool resursively=false );
		/*!
			Get the list of files in current directory
			
			\return StValueList containing files in current directory
		*/
		StValueList<StFile> getContent() const;
		
		/*!
			\deprecated use getContent() instead
		*/
		StValueList<StFile> read() const { return getContent(); }
		
		/**Set the current path to this which is passed in argument*/
		void operator=( const StString& path ){ setPath( path ); }
		/**Set the current path to this which is passed in argument*/
		void operator=( const char* path ){ setPath( path ); }
		/**Converting StDir class to StString class*/
		operator StString() const { return getPath(); }
		
	protected:
	
		StString path;	
				
};

#endif //SMART_TOOLS_OUTPUT_H
