/***************************************************************************
Copyright (c) 2006 Peter Korzuszek, Damian Rakowski, KateOS Project
 
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
 
*Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.
 
*Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS
IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE. 
****************************************************************************/

#ifndef SMART_TOOLS_ERROR_H
#define SMART_TOOLS_ERROR_H

#include "ststring.h"
#include "stvaluelist.h"

#ifndef MAX_ERROR_LOGS
	#define MAX_ERROR_LOGS 1000
#endif //MAX_ERROR_LOGS

class __StError
{
	typedef struct{
		const void* object;
		int code;
		StString humanReadable;
	} Log;
	
	public:
		/*
			Make log about error
			\param obj - object which caused error
			\param code - code of error
			\param humanReadable - human readable description of error
		*/
		void report( const void* obj, int code, const StString& humanReadable="" );
		
		/*Get the last error code of object passed by the argument*/
		int getLastCode( const void* obj );
		
		/*Get the last error code*/
		int getLastCode() { return errorLogs.last().code; }
		
		/*Get the human readable description of the last error code of object passed by the argument*/
		StString getLastHumanReadable( const void* obj );
		
		/*Get the human readable description of the last error code*/
		StString getLastHumanReadable() { return errorLogs.last().humanReadable; }
		
		void clear();
		bool hasError( const void* obj ) { return (getLastCode(obj) != -1) ? true : false; }
	
	private:
		StValueList<Log> errorLogs;
		
		__StError(){}
		friend __StError& stError();
};

__StError& stError();

/* \brief Containing logs of errors

	You could retrieve the last code error of object also in human readable mode
*/

class StError
{
	public:
		static void report( const void* obj, int code, const StString& humanReadable="" )
		{ stError().report(obj, code, humanReadable); }
		
		/*Get the last error code of object passed by the argument*/
		static int getLastCode( const void* obj ) { return stError().getLastCode(obj); }
		/*Get the last error code*/
		static int getLastCode() { return stError().getLastCode(); }
		/*Get the human readable description of the lat error code of object passed by the argument*/
		static StString getLastHumanReadable( const void* obj ) { return stError().getLastHumanReadable(obj); }
		/*Get the human readable description of the last error code*/
		static StString getLastHumanReadable() { return stError().getLastHumanReadable(); }
		
		static void clear() { stError().clear(); }
		static bool hasError( const void* obj ) { return stError().hasError(obj); }
};

#endif //SMART_TOOLS_ERROR_H
